@@ -18,7 +18,6 @@ from itertools import groupby
 import jsonschema
 from operator import itemgetter
 import six
-import time
 
 from neutronclient.common import exceptions as n_exceptions
 from oslo_concurrency import processutils
@@ -43,6 +42,7 @@ LOG = log.getLogger(__name__)
 MANDATORY_NEUTRON_EXTENSION = "subnet_allocation"
 TAG_NEUTRON_EXTENSION = "tag"
 TAG_EXT_NEUTRON_EXTENSION = "tag-ext"
+TAG_STANDARD_NEUTRON_EXTENSION = 'standard-attr-tag'
 SUBNET_POOLS_V4 = []
 SUBNET_POOLS_V6 = []
 DEFAULT_DRIVER = driver.get_driver_instance()
@@ -83,20 +83,36 @@ def check_for_neutron_ext_support():
                             .format(MANDATORY_NEUTRON_EXTENSION))
 
 
-def check_for_neutron_tag_support(ext_name):
-    """Validates tag and tag-ext extension support availability in Neutron."""
-    if ext_name == TAG_EXT_NEUTRON_EXTENSION:
-        ext_rename = "tag_ext"
-    else:
-        ext_rename = ext_name
-    setattr(app, ext_rename, True)
+def check_for_neutron_tag_support():
+    """Validates tagging extensions availability in Neutron.
+
+    Checks if either tag, tag-ext or standard-attr-tag is available and sets
+    ``app`` properties accordingly.
+    """
+
+    app.tag_ext = True
+    app.tag = True
+
+    # Check for modern extension first.
     try:
-        app.neutron.show_extension(ext_name)
-    except n_exceptions.NeutronClientException as e:
-        setattr(app, ext_rename, False)
-        if e.status_code == n_exceptions.NotFound.status_code:
-            LOG.warning("Neutron extension %s not supported. "
-                        "Continue without using them.", ext_name)
+        app.neutron.show_extension(TAG_STANDARD_NEUTRON_EXTENSION)
+    except n_exceptions.NeutronClientException:
+        pass
+    else:
+        # Okay, this means we have functionality of both tag and tag_ext,
+        # we're good to go!
+        return
+
+    # Fallback to legacy extensions.
+    for ext in (TAG_NEUTRON_EXTENSION, TAG_EXT_NEUTRON_EXTENSION):
+        try:
+            app.neutron.show_extension(ext)
+        except n_exceptions.NeutronClientException as e:
+            ext_param = ext.replace('-', '_')  # identifiers cannot have '-'
+            setattr(app, ext_param, False)
+            if e.status_code == n_exceptions.NotFound.status_code:
+                LOG.warning("Neutron extension %s not supported. "
+                            "Continue without using them.", ext)
 
 
 def load_default_subnet_pools():
@@ -118,8 +134,7 @@ def get_driver(port):
     if driver is None:
         raise exceptions.KuryrException(
             "No port driver available for VNIC type %s" % vnic_type)
-    else:
-        return driver
+    return driver
 
 
 def _cache_default_subnetpool_ids(app):
@@ -389,27 +404,6 @@ def _get_networks_by_identifier(identifier):
     return _get_networks_by_attrs(name=identifier)
 
 
-def _port_active(neutron_port_id, vif_plug_timeout):
-    port_active = False
-    tries = 0
-    while True:
-        try:
-            port = app.neutron.show_port(neutron_port_id)
-        except n_exceptions.NeutronClientException as ex:
-            LOG.error('Could not get the port %s to check '
-                      'its status', ex)
-        else:
-            if port['port']['status'] == lib_const.PORT_STATUS_ACTIVE:
-                port_active = True
-        if port_active or (vif_plug_timeout > 0 and tries >= vif_plug_timeout):
-            break
-        LOG.debug('Waiting for port %s to become ACTIVE', neutron_port_id)
-        tries += 1
-        time.sleep(1)
-
-    return port_active
-
-
 def _program_expose_ports(options, port_id):
     exposed_ports = options.get(const.DOCKER_EXPOSED_PORTS_OPTION)
     if not exposed_ports:
@@ -582,7 +576,10 @@ def _create_kuryr_subnet(pool_cidr, subnet_cidr, pool_id, network_id, gateway):
     if gateway:
         new_kuryr_subnet[0]['gateway_ip'] = gateway
 
-    app.neutron.create_subnet({'subnets': new_kuryr_subnet})
+    subnets = app.neutron.create_subnet({'subnets': new_kuryr_subnet})
+    if app.tag_ext:
+        for subnet in subnets['subnets']:
+            _neutron_subnet_add_tag(subnet['id'], pool_id)
     LOG.debug("Created kuryr subnet %s", new_kuryr_subnet)
 
 
@@ -1156,8 +1153,8 @@ def network_driver_create_endpoint():
                 LOG.error('Failed to set up the interface: %s', ex)
 
         if app.vif_plug_is_fatal:
-            port_active = _port_active(neutron_port['id'],
-                                       app.vif_plug_timeout)
+            port_active = utils.wait_for_port_active(
+                app.neutron, neutron_port['id'], app.vif_plug_timeout)
             if not port_active:
                 neutron_port_name = neutron_port['name']
                 raise exceptions.InactiveResourceException(
@@ -1421,6 +1418,9 @@ def network_driver_program_external_connectivity():
     json_data = flask.request.get_json(force=True)
     LOG.debug("Received JSON data %s for"
               " /NetworkDriver.ProgramExternalConnectivity", json_data)
+    if not cfg.CONF.process_external_connectivity:
+        return flask.jsonify(const.SCHEMA['SUCCESS'])
+
     # TODO(banix): Add support for exposed ports
     port = _get_neutron_port_from_docker_endpoint(json_data['EndpointID'])
     if port:
@@ -1443,6 +1443,9 @@ def network_driver_revoke_external_connectivity():
     json_data = flask.request.get_json(force=True)
     LOG.debug("Received JSON data %s for"
               " /NetworkDriver.RevokeExternalConnectivity", json_data)
+    if not cfg.CONF.process_external_connectivity:
+        return flask.jsonify(const.SCHEMA['SUCCESS'])
+
     # TODO(banix): Add support for removal of exposed ports
     port = _get_neutron_port_from_docker_endpoint(json_data['EndpointID'])
     if port:
@@ -1863,6 +1866,10 @@ def ipam_release_address():
     if not len(subnets):
         LOG.info("Subnet already deleted.")
         return flask.jsonify(const.SCHEMA['SUCCESS'])
+    if app.tag_ext:
+        subnets = [subnet
+                   for subnet in subnets
+                   if pool_id in subnet.get('tags') or []]
 
     iface = ipaddress.ip_interface(six.text_type(rel_address))
     rel_ip_address = six.text_type(iface.ip)
@@ -1879,10 +1886,15 @@ def ipam_release_address():
                         app.neutron.delete_port(port['id'])
             elif tags and const.KURYR_EXISTING_NEUTRON_PORT in tags:
                 updated_port = {'name': '', 'device_owner': '',
-                                'device_id': '', 'binding:host_id': ''}
-                app.neutron.update_port(port['id'], {'port': updated_port})
-                _neutron_port_remove_tag(port['id'],
-                                         const.KURYR_EXISTING_NEUTRON_PORT)
+                                'binding:host_id': ''}
+                if port['name'].startswith(port['device_id']):
+                    updated_port["device_id"] = ''
+                for subnet in subnets:
+                    if (port['fixed_ips'][0]['subnet_id'] == subnet['id']):
+                        app.neutron.update_port(
+                            port['id'], {'port': updated_port})
+                        _neutron_port_remove_tag(
+                            port['id'], const.KURYR_EXISTING_NEUTRON_PORT)
     except n_exceptions.NeutronClientException as ex:
         LOG.error("Error happened while fetching "
                   "and deleting port, %s", ex)
