@@ -19,7 +19,6 @@ from oslotest import base
 from kuryr.lib import constants as lib_const
 from kuryr.lib import utils as lib_utils
 from kuryr_libnetwork import app
-from kuryr_libnetwork import constants as const
 from kuryr_libnetwork import controllers
 from kuryr_libnetwork.port_driver import driver
 from kuryr_libnetwork import utils
@@ -267,8 +266,7 @@ class TestKuryrBase(TestCase):
         }
         if subnetpool_id:
             fake_v4_subnet['subnet'].update(subnetpool_id=subnetpool_id)
-            if not str(name).startswith(const.SUBNET_NAME_PREFIX):
-                fake_v4_subnet['subnet'].get('tags').append(subnetpool_id)
+            fake_v4_subnet['subnet'].get('tags').append(subnetpool_id)
 
         return fake_v4_subnet
 
